﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.Package;
using Microsoft.VisualStudio.TextManager.Interop;
using Microsoft.VisualStudio;
using System.Runtime.InteropServices;
using Antlr.Runtime;

namespace ManagedMyC
{
    [Guid( MyCConstants.MyCLanguageServiceGuid )]
    public class MyCLanguageService : LanguageService
    {
        MyCColorableItems _colorableItems;
        LanguagePreferences _preferences;

        public MyCLanguageService()
        {
            _colorableItems = new MyCColorableItems();
        }

        public bool UpdateDropdowns
        {
            get;
            set;
        }

        public override string GetFormatFilterList()
        {
            return "MyC File (*.myc)\n*.myc";
        }

        #region Custom colors
        public override int GetColorableItem( int index, out IVsColorableItem item )
        {
            if ( index <= _colorableItems.Count )
            {
                item = _colorableItems[index - 1];
                return VSConstants.S_OK;
            }

            item = null;
            return VSConstants.E_INVALIDARG;
        }

        public override int GetItemCount( out int count )
        {
            count = _colorableItems.Count;
            return VSConstants.S_OK;
        }
        #endregion

        #region MPF specializations
        public override LanguagePreferences GetLanguagePreferences()
        {
            if ( _preferences == null )
            {
                _preferences = new LanguagePreferences( Site, GetType().GUID, Name );
                _preferences.Init();
            }

            return _preferences;
        }

        public override TypeAndMemberDropdownBars CreateDropDownHelper( IVsTextView forView )
        {
            return new MyCTypeAndMemberDropdownBars( this );
        }

        public override Source CreateSource( IVsTextLines buffer )
        {
            return new MyCSource( this, buffer, GetColorizer( buffer ) );
        }

        public override IScanner GetScanner( IVsTextLines buffer )
        {
            return new MyCScanner( this, buffer );
        }
        #endregion

        public override void OnIdle( bool periodic )
        {
            MyCSource source = GetSource( LastActiveTextView ) as MyCSource;
            if ( source != null && source.LastParseTime >= 0x7ffff )
                source.LastParseTime = 0;

            if ( UpdateDropdowns )
                SynchronizeDropdowns();
            UpdateDropdowns = false;

            base.OnIdle( periodic );
        }

        public override AuthoringScope ParseSource( ParseRequest req )
        {
            MyCSource source = (MyCSource)GetSource( req.FileName );
            bool result = false;

            if ( req.DirtySpan.iStartIndex != req.DirtySpan.iEndIndex
                || req.DirtySpan.iStartLine != req.DirtySpan.iEndLine )
            {
                MyCErrorHandler handler = new MyCErrorHandler();
                MyCLexer lexer = new MyCLexer( req.Text );
                MyCParser parser = new MyCParser( lexer );

                lexer.Handler = handler;
                parser.Handler = handler;

                parser.Init( req );
                result = parser.Parse();

                // store the parse results
                source.ParseResult = null;
                source.Braces = parser.Braces;
                source.Types = parser.Types;
                source.Members = parser.Members;

                // for the time being, just pull errors back from the error handler
                if ( handler.ErrNum > 0 )
                {
                    foreach ( Error error in handler.SortedErrorList() )
                    {
                        TextSpan span = new TextSpan()
                        {
                            iStartLine = error.line - 1,
                            iStartIndex = error.column,
                            iEndLine = error.line - 1,
                            iEndIndex = error.column + error.length
                        };

                        req.Sink.AddError( req.FileName, error.message, span, Severity.Error );
                    }
                }
            }

            switch ( req.Reason )
            {
            case ParseReason.Check:
            case ParseReason.HighlightBraces:
            case ParseReason.MatchBraces:
            case ParseReason.MemberSelectAndHighlightBraces:
                // send matches to sink
                // this should (probably?) be filtered on req.Line / col
                if ( source.Braces != null )
                {
                    foreach ( TextSpan[] brace in source.Braces )
                    {
                        if ( brace.Length == 2 )
                            req.Sink.MatchPair( brace[0], brace[1], 1 );
                        else if ( brace.Length >= 3 )
                            req.Sink.MatchTriple( brace[0], brace[1], brace[2], 1 );
                    }
                }

                // make sure the dropdowns are updated after a parse
                UpdateDropdowns = true;

                break;

            default:
                break;
            }

            return new MyCAuthoringScope( source.ParseResult );
        }

        public override string Name
        {
            get
            {
                return MyCConstants.LanguageName;
            }
        }
    }
}
