using Microsoft.VisualStudio.Package;
using Microsoft.VisualStudio.TextManager.Interop;
using System.Collections.Generic;
using Antlr.Runtime;
using System;
using System.Linq;
using Antlr.Runtime.Tree;

namespace ManagedMyC
{
    partial class MyCParser
    {
        public MyCParser( MyCLexer lexer )
            : this( new CommonTokenStream( lexer ) )
        {
        }

        public MyCErrorHandler Handler
        {
            get;
            set;
        }
        public ParseRequest Request
        {
            get;
            private set;
        }
        public IList<TextSpan[]> Braces
        {
            get;
            private set;
        }
        public AuthoringSink Sink
        {
            get
            {
                return Request.Sink;
            }
        }

        public IList<DropDownMember> Types
        {
            get
            {
                if ( Walker == null || Walker.UnsortedTypes == null )
                    return new DropDownMember[0];
                return Walker.UnsortedTypes.OrderBy( type => PositionIndicator( type ) ).ToArray();
            }
        }

        public IList<DropDownMember> Members
        {
            get
            {
                if ( Walker == null || Walker.UnsortedMembers == null )
                    return new DropDownMember[0];
                return Walker.UnsortedMembers.OrderBy( member => PositionIndicator( member ) ).ToArray();
            }
        }

        public MyCWalker Walker
        {
            get;
            private set;
        }

        public void Init( ParseRequest request )
        {
            Request = request;
            Braces = new List<TextSpan[]>();
        }

        public bool Parse()
        {
            program_return result = program();

            Walker = new MyCWalker( this, (CommonTree)result.Tree );
            Walker.WalkAST();

            return true;
        }

        static int PositionIndicator( DropDownMember member )
        {
            return ( member.Span.iStartLine << 10 ) + member.Span.iStartIndex;
        }

        public override void ReportError( RecognitionException e )
        {
            System.Diagnostics.Debug.Write( GetErrorMessage( e, MyCParser.tokenNames ) );
            base.ReportError( e );
        }

        internal void CallHdlr( string message, IToken location )
        {
            TextSpan span = ToTextSpan( location );
            Handler.AddError( message, span.iStartLine, span.iStartIndex, Math.Max( 1, span.iEndIndex - span.iStartIndex ) );
        }

        public void Match( IToken lh, IToken rh )
        {
            DefineMatch( ToTextSpan( lh ), ToTextSpan( rh ) );
        }
        public void DefineMatch( int priority, params TextSpan[] locations )
        {
            if ( locations != null && locations.Length > 0 )
                Braces.Add( locations.Clone() as TextSpan[] );
        }
        public void DefineMatch( params TextSpan[] locations )
        {
            DefineMatch( 0, locations );
        }

        public void Region( IToken lh, IToken rh )
        {
            DefineRegion( TextSpanHelper.Merge( ToTextSpan( lh ), ToTextSpan( rh ) ) );
        }
        public void Region( IToken token )
        {
            DefineRegion( ToTextSpan( token ) );
        }
        public void DefineRegion( TextSpan span )
        {
            Sink.ProcessHiddenRegions = true;
            Sink.AddHiddenRegion( span );
        }

        public void ReportError( TextSpan span, string message, Severity severity )
        {
            Sink.AddError( Request.FileName, message, span, severity );
        }

        public void ReportError( TextSpan span, string message )
        {
            ReportError( span, message, Severity.Error );
        }
        public void ReportFatal( TextSpan span, string message )
        {
            ReportError( span, message, Severity.Fatal );
        }
        public void ReportWarning( TextSpan span, string message )
        {
            ReportError( span, message, Severity.Warning );
        }
        public void ReportHint( TextSpan span, string message )
        {
            ReportError( span, message, Severity.Hint );
        }

        public static TextSpan ToTextSpan( IToken token )
        {
            if ( token == null )
                return new TextSpan();

            string tokentext = token.Text ?? string.Empty;

            return new TextSpan()
            {
                iStartLine = token.Line - 1,
                iStartIndex = token.CharPositionInLine,
                iEndLine = token.Line - 1,
                iEndIndex = token.CharPositionInLine + tokentext.Length
            };
        }
    }
}
