﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.Package;
using Microsoft.VisualStudio.TextManager.Interop;

using ArrayList = System.Collections.ArrayList;

namespace ManagedMyC
{

    public class MyCTypeAndMemberDropdownBars : TypeAndMemberDropdownBars
    {
        public MyCTypeAndMemberDropdownBars( MyCLanguageService languageService )
            : base( languageService )
        {
        }

        public override bool OnSynchronizeDropdowns( LanguageService languageService, IVsTextView textView, int line, int col, ArrayList dropDownTypes, ArrayList dropDownMembers, ref int selectedTypeIndex, ref int selectedMemberIndex )
        {
            MyCSource source = languageService.GetSource( textView ) as MyCSource;
            if ( source == null )
                return false;

            DropDownMember selectedType = null;
            DropDownMember selectedMember = null;

            if ( selectedTypeIndex >= 0 && dropDownTypes.Count > selectedTypeIndex )
                selectedType = dropDownTypes[selectedTypeIndex] as DropDownMember;

            if ( selectedMemberIndex >= 0 && dropDownMembers.Count > selectedMemberIndex )
                selectedMember = dropDownMembers[selectedMemberIndex] as DropDownMember;

            IEnumerable<DropDownMember> types = source.GetDropDownTypes();
            if ( types == null )
                return false;

            if ( source.TypesUpdated )
            {
                dropDownTypes.Clear();
                dropDownTypes.AddRange( types.ToList() );
                source.TypesUpdated = false;
            }

            DropDownMember typeScope = ( from type in types
                                         where TextSpanHelper.ContainsInclusive( type.Span, line, col )
                                         orderby ( type.Span.iStartLine << 16 ) + type.Span.iStartIndex
                                         select type )
                                       .LastOrDefault();

            if ( typeScope != null )
                selectedTypeIndex = dropDownTypes.IndexOf( typeScope );
            else if ( selectedTypeIndex < 0 || selectedTypeIndex >= dropDownTypes.Count )
                selectedTypeIndex = source.GetDefaultType( types );

            IEnumerable<TextSpan> excludeSpans = null;
            if ( typeScope != null )
            {
                // exclude sub-spans so members of nested types don't show as members of the current type
                excludeSpans = from type in types
                               where TextSpanHelper.IsEmbedded( type.Span, typeScope.Span )
                               where !TextSpanHelper.IsSameSpan( typeScope.Span, type.Span )
                               select type.Span;
            }
            else
            {
                excludeSpans = Enumerable.Empty<TextSpan>();
            }

            IEnumerable<DropDownMember> members = source.GetDropDownMembers( typeScope, excludeSpans );
            if ( members == null )
                return false;

            if ( source.MembersUpdated || selectedType != typeScope )
            {
                dropDownMembers.Clear();
                dropDownMembers.AddRange( members.ToList() );
                source.MembersUpdated = false;
            }

            DropDownMember memberScope = ( from member in members
                                           where TextSpanHelper.ContainsInclusive( member.Span, line, col )
                                           orderby ( member.Span.iStartLine << 16 ) + member.Span.iStartIndex
                                           select member )
                                         .LastOrDefault();

            if ( memberScope != null )
                selectedMemberIndex = dropDownMembers.IndexOf( memberScope );
            else if ( selectedMemberIndex < 0 || selectedMemberIndex >= dropDownMembers.Count )
                selectedMemberIndex = source.GetDefaultMember( selectedType, members );

            if ( selectedTypeIndex >= 0 )
            {
                // gray out the displayed type if the cursor is no longer in its span
                ( (DropDownMember)dropDownTypes[selectedTypeIndex] ).FontAttr =
                    ( typeScope == null ) ? DROPDOWNFONTATTR.FONTATTR_GRAY : DROPDOWNFONTATTR.FONTATTR_PLAIN;
            }
            if ( selectedMemberIndex >= 0 )
            {
                // gray out the displayed member if the cursor is no longer in its span
                ( (DropDownMember)dropDownMembers[selectedMemberIndex] ).FontAttr =
                    ( memberScope == null ) ? DROPDOWNFONTATTR.FONTATTR_GRAY : DROPDOWNFONTATTR.FONTATTR_PLAIN;
            }

            return true;
        }

    }

}
