using System.Collections.Generic;
using Antlr.Runtime;
using Antlr.Runtime.Tree;
using Microsoft.VisualStudio.Package;
using Microsoft.VisualStudio.TextManager.Interop;

namespace ManagedMyC
{
    partial class MyCWalker
    {
        public MyCWalker( MyCParser parser, CommonTree tree )
            : this( new CommonTreeNodeStream( tree ) )
        {
            ( (CommonTreeNodeStream)input ).TokenStream = parser.TokenStream;
        }

        public List<DropDownMember> UnsortedTypes
        {
            get;
            private set;
        }
        public List<DropDownMember> UnsortedMembers
        {
            get;
            private set;
        }

        public void WalkAST()
        {
            UnsortedTypes = new List<DropDownMember>();
            UnsortedMembers = new List<DropDownMember>();
            program();
        }

        void AddType( string name, IToken start, IToken stop, IconImageIndex glyph )
        {
            if ( name == null || start == null || stop == null )
                return;

            DropDownMember type = new DropDownMember(
                name,
                TextSpanHelper.Merge( MyCParser.ToTextSpan( start ), MyCParser.ToTextSpan( stop ) ),
                (int)glyph,
                DROPDOWNFONTATTR.FONTATTR_PLAIN );

            UnsortedTypes.Add( type );
        }
        void AddMember( string name, IToken start, IToken stop, IconImageIndex glyph )
        {
            if ( name == null || start == null || stop == null )
                return;

            DropDownMember member = new DropDownMember(
                name,
                TextSpanHelper.Merge( MyCParser.ToTextSpan( start ), MyCParser.ToTextSpan( stop ) ),
                (int)glyph,
                DROPDOWNFONTATTR.FONTATTR_PLAIN );

            UnsortedMembers.Add( member );
        }
    }
}
